<div class="{% spaceless %}form-field form-field-type-{{model_field.type}} 
{% if model_field.extra_css_classes %}{{model_field.extra_css_classes}} {% endif %}
{% endspaceless %}">

<h3>{{model_field.title|safe}}</h3>
<hr />

</div>