{% load form_tags %}

<form action="{% if form_entry_object %}{{form_entry_object.get_absolute_url}}{% else %}{{object.get_form_action}}{% endif %}" method="post" {% if form_entry_object.has_multipart %}enctype="multipart/form-data"{% endif %} class="form-name-{{form_schema.slug}}">
    {% csrf_token %}
    
    <input  type="hidden" name="{{ form.form_schema.name }}" id="{{ form.form_schema.auto_id }}" value="{{ form.form_schema.value|default_if_none:'' }}" />

    {% for field in form %}
    
    {% if field.field.field_model %}
    
    {% get_rendered_field field as rendered_field %}
    {{rendered_field|safe}}
    
    {% endif %}
    
    {% endfor %}

    <footer>
    	<input type="submit" value="{% firstof form_schema.submit_label 'Submit' %}" />
    </footer>
</form>