#Font Awesome 4.3.0

ICON_CHOICES = (
	('glass','Glass'),
	('music', 'Music'),
	('search', 'Search'),
	('envelope-o', 'Envelope O'),
	('heart', 'Heart'),
	('star', 'Star'),
	('star-o', 'Star o'),
	('user', 'User'),
	('film', 'Film'),
	('th-large', 'Th Large'),
	('th', 'Th'),
	('th-list', 'Th List'),
	('check', 'Check'),
	('remove', 'Remove'),
	('close', 'Close'),
	('times', 'Times'),
	('search-plus', 'Search Plus'),
	('search-minus', 'Search Minus'),
	('power-off', 'Power Off'),
	('signal', 'Signal'),
	('gear', 'Gear'),
	('cog', 'Cog'),
	('trash-o', 'Trash o'),
	('home', 'Home'),
	('file-o', 'File o'),
	('clock-o', 'Clock o'),
	('road', 'Road'),
	('download', 'Download'),
	('arrow-circle-o-down', 'Arrow circle-o Down'),
	('arrow-circle-o-up', 'Arrow circle-o Up'),
	('inbox', 'Inbox'),
	('play-circle-o', 'Play circle o'),
	('rotate-right', 'Rotate Right'),
	('repeat', 'Repeat'),
	('refresh', 'Refresh'),
	('list-alt', 'List Alt'),
	('lock', 'Lock'),
	('flag', 'Flag'),
	('headphones', 'Headphones'),
	('volume-off', 'Volume Off'),
	('volume-down', 'Volume Down'),
	('volume-up', 'Volume Up'),
	('qrcode', 'Qrcode'),
	('barcode', 'Barcode'),
	('tag', 'Tag'),
	('tags', 'Tags'),
	('book', 'Book'),
	('bookmark', 'Bookmark'),
	('print', 'Print'),
	('camera', 'Camera'),
	('font', 'Font'),
	('bold', 'Bold'),
	('italic', 'Italic'),
	('text-height', 'Text Height'),
	('text-width', 'Text Width'),
	('align-left', 'Align Left'),
	('align-center', 'Align Center'),
	('align-right', 'Align Right'),
	('align-justify', 'Align Justify'),
	('list', 'List'),
	('dedent', 'Dedent'),
	('outdent', 'Outdent'),
	('indent', 'Indent'),
	('video-camera', 'Video Camera'),
	('photo', 'Photo'),
	('image', 'Image'),
	('picture-o', 'Picture o'),
	('pencil', 'Pencil'),
	('map-marker', 'Map Marker'),
	('adjust', 'Adjust'),
	('tint', 'Tint'),
	('edit', 'Edit'),
	('pencil-square-o', 'Pencil square o'),
	('share-square-o', 'Share square o'),
	('check-square-o', 'Check square o'),
	('arrows', 'Arrows'),
	('step-backward', 'Step Backward'),
	('fast-backward', 'Fast Backward'),
	('backward', 'Backward'),
	('play', 'Play'),
	('pause', 'Pause'),
	('stop', 'Stop'),
	('forward', 'Forward'),
	('fast-forward', 'Fast Forward'),
	('step-forward', 'Step Forward'),
	('eject', 'Eject'),
	('chevron-left', 'Chevron Left'),
	('chevron-right', 'Chevron Right'),
	('plus-circle', 'Plus Circle'),
	('minus-circle', 'Minus Circle'),
	('times-circle', 'Times Circle'),
	('check-circle', 'Check Circle'),
	('question-circle', 'Question Circle'),
	('info-circle', 'Info Circle'),
	('crosshairs', 'Crosshairs'),
	('times-circle-o', 'Times circle o'),
	('check-circle-o', 'Check circle o'),
	('ban', 'Ban'),
	('arrow-left', 'Arrow Left'),
	('arrow-right', 'Arrow Right'),
	('arrow-up', 'Arrow Up'),
	('arrow-down', 'Arrow Down'),
	('mail-forward', 'Mail Forward'),
	('share', 'Share'),
	('expand', 'Expand'),
	('compress', 'Compress'),
	('plus', 'Plus'),
	('minus', 'Minus'),
	('asterisk', 'Asterisk'),
	('exclamation-circle', 'Exclamation Circle'),
	('gift', 'Gift'),
	('leaf', 'Leaf'),
	('fire', 'Fire'),
	('eye', 'Eye'),
	('eye-slash', 'Eye Slash'),
	('warning', 'Warning'),
	('exclamation-triangle', 'Exclamation Triangle'),
	('plane', 'Plane'),
	('calendar', 'Calendar'),
	('random', 'Random'),
	('comment', 'Comment'),
	('magnet', 'Magnet'),
	('chevron-up', 'Chevron Up'),
	('chevron-down', 'Chevron Down'),
	('retweet', 'Retweet'),
	('shopping-cart', 'Shopping Cart'),
	('folder', 'Folder'),
	('folder-open', 'Folder Open'),
	('arrows-v', 'Arrows v'),
	('arrows-h', 'Arrows h'),
	('bar-chart-o', 'Bar chart o'),
	('bar-chart', 'Bar Chart'),
	('twitter-square', 'Twitter Square'),
	('facebook-square', 'Facebook Square'),
	('camera-retro', 'Camera Retro'),
	('key', 'Key'),
	('gears', 'Gears'),
	('cogs', 'Cogs'),
	('comments', 'Comments'),
	('thumbs-o-up', 'Thumbs o Up'),
	('thumbs-o-down', 'Thumbs o Down'),
	('star-half', 'Star Half'),
	('heart-o', 'Heart o'),
	('sign-out', 'Sign Out'),
	('linkedin-square', 'Linkedin Square'),
	('thumb-tack', 'Thumb Tack'),
	('external-link', 'External Link'),
	('sign-in', 'Sign In'),
	('trophy', 'Trophy'),
	('github-square', 'Github Square'),
	('upload', 'Upload'),
	('lemon-o', 'Lemon o'),
	('phone', 'Phone'),
	('square-o', 'Square o'),
	('bookmark-o', 'Bookmark o'),
	('phone-square', 'Phone Square'),
	('twitter', 'Twitter'),
	('facebook-f', 'Facebook f'),
	('facebook', 'Facebook'),
	('github', 'Github'),
	('unlock', 'Unlock'),
	('credit-card', 'Credit Card'),
	('rss', 'Rss'),
	('hdd-o', 'Hdd o'),
	('bullhorn', 'Bullhorn'),
	('bell', 'Bell'),
	('certificate', 'Certificate'),
	('hand-o-right', 'Hand o Right'),
	('hand-o-left', 'Hand o Left'),
	('hand-o-up', 'Hand o Up'),
	('hand-o-down', 'Hand o Down'),
	('arrow-circle-left', 'Arrow circle Left'),
	('arrow-circle-right', 'Arrow circle Right'),
	('arrow-circle-up', 'Arrow circle Up'),
	('arrow-circle-down', 'Arrow circle Down'),
	('globe', 'Globe'),
	('wrench', 'Wrench'),
	('tasks', 'Tasks'),
	('filter', 'Filter'),
	('briefcase', 'Briefcase'),
	('arrows-alt', 'Arrows Alt'),
	('group', 'Group'),
	('users', 'Users'),
	('chain', 'Chain'),
	('link', 'Link'),
	('cloud', 'Cloud'),
	('flask', 'Flask'),
	('cut', 'Cut'),
	('scissors', 'Scissors'),
	('copy', 'Copy'),
	('files-o', 'Files o'),
	('paperclip', 'Paperclip'),
	('save', 'Save'),
	('floppy-o', 'Floppy o'),
	('square', 'Square'),
	('navicon', 'Navicon'),
	('reorder', 'Reorder'),
	('bars', 'Bars'),
	('list-ul', 'List Ul'),
	('list-ol', 'List Ol'),
	('strikethrough', 'Strikethrough'),
	('underline', 'Underline'),
	('table', 'Table'),
	('magic', 'Magic'),
	('truck', 'Truck'),
	('pinterest', 'Pinterest'),
	('pinterest-square', 'Pinterest Square'),
	('google-plus-square', 'Google plus Square'),
	('google-plus', 'Google Plus'),
	('money', 'Money'),
	('caret-down', 'Caret Down'),
	('caret-up', 'Caret Up'),
	('caret-left', 'Caret Left'),
	('caret-right', 'Caret Right'),
	('columns', 'Columns'),
	('unsorted', 'Unsorted'),
	('sort', 'Sort'),
	('sort-down', 'Sort Down'),
	('sort-desc', 'Sort Desc'),
	('sort-up', 'Sort Up'),
	('sort-asc', 'Sort Asc'),
	('envelope', 'Envelope'),
	('linkedin', 'Linkedin'),
	('rotate-left', 'Rotate Left'),
	('undo', 'Undo'),
	('legal', 'Legal'),
	('gavel', 'Gavel'),
	('dashboard', 'Dashboard'),
	('tachometer', 'Tachometer'),
	('comment-o', 'Comment o'),
	('comments-o', 'Comments o'),
	('flash', 'Flash'),
	('bolt', 'Bolt'),
	('sitemap', 'Sitemap'),
	('umbrella', 'Umbrella'),
	('paste', 'Paste'),
	('clipboard', 'Clipboard'),
	('lightbulb-o', 'Lightbulb o'),
	('exchange', 'Exchange'),
	('cloud-download', 'Cloud Download'),
	('cloud-upload', 'Cloud Upload'),
	('user-md', 'User Md'),
	('stethoscope', 'Stethoscope'),
	('suitcase', 'Suitcase'),
	('bell-o', 'Bell o'),
	('coffee', 'Coffee'),
	('cutlery', 'Cutlery'),
	('file-text-o', 'File text o'),
	('building-o', 'Building o'),
	('hospital-o', 'Hospital o'),
	('ambulance', 'Ambulance'),
	('medkit', 'Medkit'),
	('fighter-jet', 'Fighter Jet'),
	('beer', 'Beer'),
	('h-square', 'h Square'),
	('plus-square', 'Plus Square'),
	('angle-double-left', 'Angle double Left'),
	('angle-double-right', 'Angle double Right'),
	('angle-double-up', 'Angle double Up'),
	('angle-double-down', 'Angle double Down'),
	('angle-left', 'Angle Left'),
	('angle-right', 'Angle Right'),
	('angle-up', 'Angle Up'),
	('angle-down', 'Angle Down'),
	('desktop', 'Desktop'),
	('laptop', 'Laptop'),
	('tablet', 'Tablet'),
	('mobile-phone', 'Mobile Phone'),
	('mobile', 'Mobile'),
	('circle-o', 'Circle o'),
	('quote-left', 'Quote Left'),
	('quote-right', 'Quote Right'),
	('spinner', 'Spinner'),
	('circle', 'Circle'),
	('mail-reply', 'Mail Reply'),
	('reply', 'Reply'),
	('github-alt', 'Github Alt'),
	('folder-o', 'Folder o'),
	('folder-open-o', 'Folder open o'),
	('smile-o', 'Smile o'),
	('frown-o', 'Frown o'),
	('meh-o', 'Meh o'),
	('gamepad', 'Gamepad'),
	('keyboard-o', 'Keyboard o'),
	('flag-o', 'Flag o'),
	('flag-checkered', 'Flag Checkered'),
	('terminal', 'Terminal'),
	('code', 'Code'),
	('mail-reply-all', 'Mail reply All'),
	('reply-all', 'Reply All'),
	('star-half-empty', 'Star half Empty'),
	('star-half-full', 'Star half Full'),
	('star-half-o', 'Star half o'),
	('location-arrow', 'Location Arrow'),
	('crop', 'Crop'),
	('code-fork', 'Code Fork'),
	('unlink', 'Unlink'),
	('chain-broken', 'Chain Broken'),
	('question', 'Question'),
	('info', 'Info'),
	('exclamation', 'Exclamation'),
	('superscript', 'Superscript'),
	('subscript', 'Subscript'),
	('eraser', 'Eraser'),
	('puzzle-piece', 'Puzzle Piece'),
	('microphone', 'Microphone'),
	('microphone-slash', 'Microphone Slash'),
	('shield', 'Shield'),
	('calendar-o', 'Calendar o'),
	('fire-extinguisher', 'Fire Extinguisher'),
	('rocket', 'Rocket'),
	('maxcdn', 'Maxcdn'),
	('chevron-circle-left', 'Chevron circle Left'),
	('chevron-circle-right', 'Chevron circle Right'),
	('chevron-circle-up', 'Chevron circle Up'),
	('chevron-circle-down', 'Chevron circle Down'),
	('html5', 'Html5'),
	('css3', 'Css3'),
	('anchor', 'Anchor'),
	('unlock-alt', 'Unlock Alt'),
	('bullseye', 'Bullseye'),
	('ellipsis-h', 'Ellipsis h'),
	('ellipsis-v', 'Ellipsis v'),
	('rss-square', 'Rss Square'),
	('play-circle', 'Play Circle'),
	('ticket', 'Ticket'),
	('minus-square', 'Minus Square'),
	('minus-square-o', 'Minus square o'),
	('level-up', 'Level Up'),
	('level-down', 'Level Down'),
	('check-square', 'Check Square'),
	('pencil-square', 'Pencil Square'),
	('external-link-square', 'External link Square'),
	('share-square', 'Share Square'),
	('compass', 'Compass'),
	('toggle-down', 'Toggle Down'),
	('caret-square-o-down', 'Caret square-o Down'),
	('toggle-up', 'Toggle Up'),
	('caret-square-o-up', 'Caret square-o Up'),
	('toggle-right', 'Toggle Right'),
	('caret-square-o-right', 'Caret square-o Right'),
	('euro', 'Euro'),
	('eur', 'Eur'),
	('gbp', 'Gbp'),
	('dollar', 'Dollar'),
	('usd', 'Usd'),
	('rupee', 'Rupee'),
	('inr', 'Inr'),
	('cny', 'Cny'),
	('rmb', 'Rmb'),
	('yen', 'Yen'),
	('jpy', 'Jpy'),
	('ruble', 'Ruble'),
	('rouble', 'Rouble'),
	('rub', 'Rub'),
	('won', 'Won'),
	('krw', 'Krw'),
	('bitcoin', 'Bitcoin'),
	('btc', 'Btc'),
	('file', 'File'),
	('file-text', 'File Text'),
	('sort-alpha-asc', 'Sort alpha Asc'),
	('sort-alpha-desc', 'Sort alpha Desc'),
	('sort-amount-asc', 'Sort amount Asc'),
	('sort-amount-desc', 'Sort amount Desc'),
	('sort-numeric-asc', 'Sort numeric Asc'),
	('sort-numeric-desc', 'Sort numeric Desc'),
	('thumbs-up', 'Thumbs Up'),
	('thumbs-down', 'Thumbs Down'),
	('youtube-square', 'Youtube Square'),
	('youtube', 'Youtube'),
	('xing', 'Xing'),
	('xing-square', 'Xing Square'),
	('youtube-play', 'Youtube Play'),
	('dropbox', 'Dropbox'),
	('stack-overflow', 'Stack Overflow'),
	('instagram', 'Instagram'),
	('flickr', 'Flickr'),
	('adn', 'Adn'),
	('bitbucket', 'Bitbucket'),
	('bitbucket-square', 'Bitbucket Square'),
	('tumblr', 'Tumblr'),
	('tumblr-square', 'Tumblr Square'),
	('long-arrow-down', 'Long arrow Down'),
	('long-arrow-up', 'Long arrow Up'),
	('long-arrow-left', 'Long arrow Left'),
	('long-arrow-right', 'Long arrow Right'),
	('apple', 'Apple'),
	('windows', 'Windows'),
	('android', 'Android'),
	('linux', 'Linux'),
	('dribbble', 'Dribbble'),
	('skype', 'Skype'),
	('foursquare', 'Foursquare'),
	('trello', 'Trello'),
	('female', 'Female'),
	('male', 'Male'),
	('gittip', 'Gittip'),
	('gratipay', 'Gratipay'),
	('sun-o', 'Sun o'),
	('moon-o', 'Moon o'),
	('archive', 'Archive'),
	('bug', 'Bug'),
	('vk', 'Vk'),
	('weibo', 'Weibo'),
	('renren', 'Renren'),
	('pagelines', 'Pagelines'),
	('stack-exchange', 'Stack Exchange'),
	('arrow-circle-o-right', 'Arrow circle-o Right'),
	('arrow-circle-o-left', 'Arrow circle-o Left'),
	('toggle-left', 'Toggle Left'),
	('caret-square-o-left', 'Caret square-o Left'),
	('dot-circle-o', 'Dot circle o'),
	('wheelchair', 'Wheelchair'),
	('vimeo-square', 'Vimeo Square'),
	('turkish-lira', 'Turkish Lira'),
	('try', 'Try'),
	('plus-square-o', 'Plus square o'),
	('space-shuttle', 'Space Shuttle'),
	('slack', 'Slack'),
	('envelope-square', 'Envelope Square'),
	('wordpress', 'Wordpress'),
	('openid', 'Openid'),
	('institution', 'Institution'),
	('bank', 'Bank'),
	('university', 'University'),
	('mortar-board', 'Mortar Board'),
	('graduation-cap', 'Graduation Cap'),
	('yahoo', 'Yahoo'),
	('google', 'Google'),
	('reddit', 'Reddit'),
	('reddit-square', 'Reddit Square'),
	('stumbleupon-circle', 'Stumbleupon Circle'),
	('stumbleupon', 'Stumbleupon'),
	('delicious', 'Delicious'),
	('digg', 'Digg'),
	('pied-piper', 'Pied Piper'),
	('pied-piper-alt', 'Pied piper Alt'),
	('drupal', 'Drupal'),
	('joomla', 'Joomla'),
	('language', 'Language'),
	('fax', 'Fax'),
	('building', 'Building'),
	('child', 'Child'),
	('paw', 'Paw'),
	('spoon', 'Spoon'),
	('cube', 'Cube'),
	('cubes', 'Cubes'),
	('behance', 'Behance'),
	('behance-square', 'Behance Square'),
	('steam', 'Steam'),
	('steam-square', 'Steam Square'),
	('recycle', 'Recycle'),
	('automobile', 'Automobile'),
	('car', 'Car'),
	('cab', 'Cab'),
	('taxi', 'Taxi'),
	('tree', 'Tree'),
	('spotify', 'Spotify'),
	('deviantart', 'Deviantart'),
	('soundcloud', 'Soundcloud'),
	('database', 'Database'),
	('file-pdf-o', 'File pdf o'),
	('file-word-o', 'File word o'),
	('file-excel-o', 'File excel o'),
	('file-powerpoint-o', 'File powerpoint o'),
	('file-photo-o', 'File photo o'),
	('file-picture-o', 'File picture o'),
	('file-image-o', 'File image o'),
	('file-zip-o', 'File zip o'),
	('file-archive-o', 'File archive o'),
	('file-sound-o', 'File sound o'),
	('file-audio-o', 'File audio o'),
	('file-movie-o', 'File movie o'),
	('file-video-o', 'File video o'),
	('file-code-o', 'File code o'),
	('vine', 'Vine'),
	('codepen', 'Codepen'),
	('jsfiddle', 'Jsfiddle'),
	('life-bouy', 'Life Bouy'),
	('life-buoy', 'Life Buoy'),
	('life-saver', 'Life Saver'),
	('support', 'Support'),
	('life-ring', 'Life Ring'),
	('circle-o-notch', 'Circle o Notch'),
	('ra', 'Ra'),
	('rebel', 'Rebel'),
	('ge', 'Ge'),
	('empire', 'Empire'),
	('git-square', 'Git Square'),
	('git', 'Git'),
	('hacker-news', 'Hacker News'),
	('tencent-weibo', 'Tencent Weibo'),
	('qq', 'Qq'),
	('wechat', 'Wechat'),
	('weixin', 'Weixin'),
	('send', 'Send'),
	('paper-plane', 'Paper Plane'),
	('send-o', 'Send o'),
	('paper-plane-o', 'Paper plane o'),
	('history', 'History'),
	('genderless', 'Genderless'),
	('circle-thin', 'Circle Thin'),
	('header', 'Header'),
	('paragraph', 'Paragraph'),
	('sliders', 'Sliders'),
	('share-alt', 'Share Alt'),
	('share-alt-square', 'Share alt Square'),
	('bomb', 'Bomb'),
	('soccer-ball-o', 'Soccer ball o'),
	('futbol-o', 'Futbol o'),
	('tty', 'Tty'),
	('binoculars', 'Binoculars'),
	('plug', 'Plug'),
	('slideshare', 'Slideshare'),
	('twitch', 'Twitch'),
	('yelp', 'Yelp'),
	('newspaper-o', 'Newspaper o'),
	('wifi', 'Wifi'),
	('calculator', 'Calculator'),
	('paypal', 'Paypal'),
	('google-wallet', 'Google Wallet'),
	('cc-visa', 'Cc Visa'),
	('cc-mastercard', 'Cc Mastercard'),
	('cc-discover', 'Cc Discover'),
	('cc-amex', 'Cc Amex'),
	('cc-paypal', 'Cc Paypal'),
	('cc-stripe', 'Cc Stripe'),
	('bell-slash', 'Bell Slash'),
	('bell-slash-o', 'Bell slash o'),
	('trash', 'Trash'),
	('copyright', 'Copyright'),
	('at', 'At'),
	('eyedropper', 'Eyedropper'),
	('paint-brush', 'Paint Brush'),
	('birthday-cake', 'Birthday Cake'),
	('area-chart', 'Area Chart'),
	('pie-chart', 'Pie Chart'),
	('line-chart', 'Line Chart'),
	('lastfm', 'Lastfm'),
	('lastfm-square', 'Lastfm Square'),
	('toggle-off', 'Toggle Off'),
	('toggle-on', 'Toggle On'),
	('bicycle', 'Bicycle'),
	('bus', 'Bus'),
	('ioxhost', 'Ioxhost'),
	('angellist', 'Angellist'),
	('cc', 'Cc'),
	('shekel', 'Shekel'),
	('sheqel', 'Sheqel'),
	('ils', 'Ils'),
	('meanpath', 'Meanpath'),
	('buysellads', 'Buysellads'),
	('connectdevelop', 'Connectdevelop'),
	('dashcube', 'Dashcube'),
	('forumbee', 'Forumbee'),
	('leanpub', 'Leanpub'),
	('sellsy', 'Sellsy'),
	('shirtsinbulk', 'Shirtsinbulk'),
	('simplybuilt', 'Simplybuilt'),
	('skyatlas', 'Skyatlas'),
	('cart-plus', 'Cart Plus'),
	('cart-arrow-down', 'Cart arrow Down'),
	('diamond', 'Diamond'),
	('ship', 'Ship'),
	('user-secret', 'User Secret'),
	('motorcycle', 'Motorcycle'),
	('street-view', 'Street View'),
	('heartbeat', '\U$1\L$2'),
	('venus', 'Venus'),
	('mars', 'Mars'),
	('mercury', 'Mercury'),
	('transgender', 'Transgender'),
	('transgender-alt', 'Transgender Alt'),
	('venus-double', 'Venus Double'),
	('mars-double', 'Mars Double'),
	('venus-mars', 'Venus Mars'),
	('mars-stroke', 'Mars Stroke'),
	('mars-stroke-v', 'Mars stroke v'),
	('mars-stroke-h', 'Mars stroke h'),
	('neuter', 'Neuter'),
	('facebook-official', 'Facebook Official'),
	('pinterest-p', 'Pinterest p'),
	('whatsapp', 'Whatsapp'),
	('server', 'Server'),
	('user-plus', 'User Plus'),
	('user-times', 'User Times'),
	('hotel', 'Hotel'),
	('bed', 'Bed'),
	('viacoin', 'Viacoin'),
	('train', 'Train'),
	('subway', 'Subway'),
	('medium','Medium')
)