{% comment %}{{model_field.type}} - {{model_field.title}}{% endcomment %}

{% if model_field.type == 'text-field' or model_field.type == 'email-field' or model_field.type == 'url-field' or model_field.type == 'integer-field' or model_field.type == 'number-field' or model_field.type == 'date' or model_field.type == 'time' or model_field.type == 'date-time' or model_field.type == 'comma-separated-list' or model_field.type == 'score' or model_field.type == 'range' or model_field.type == 'number-slider' or model_field.type == 'password' %}

{% include 'form/partials/text_field.html' %}

{% elif model_field.type == 'text-area' %}

{% include 'form/partials/text_area.html' %}

{% elif model_field.type == 'boolean-checkboxes' or model_field.type == 'boolean-toggle' %}

{% include 'form/partials/check_box.html' %}

{% elif model_field.type == 'select-dropdown' %}

{% include 'form/partials/select.html' %}

{% elif model_field.type == 'select-radio-buttons' or model_field.type == 'select-buttons' %}

{% include 'form/partials/input_group.html' with radio=True %}

{% elif model_field.type == 'select-multiple-checkboxes' or model_field.type == 'select-multiple-buttons' %}

{% include 'form/partials/input_group.html' %}

{% elif model_field.type == 'select-multiple-autosuggest' or model_field.type == 'select-multiple-horizontal'  %}

{% include 'form/partials/select.html' with multiple=True %}

{% elif model_field.type == 'file' %}

{% include 'form/partials/file.html' %}

{% elif model_field.type == 'secure-file' %}

{% include 'form/partials/file.html' with secure=True %}

{% elif model_field.type == 'image' %}

{% include 'form/partials/file.html' %}

{% elif model_field.type == 'select-image' %}

{% include 'form/partials/select_image.html' %}

{% elif model_field.type == 'select-multiple-images' %}

{% include 'form/partials/select_image.html' with multiple=True %}

{% elif model_field.type == 'form-instructions' %}

{% include 'form/partials/instructions.html' %}

{% elif model_field.type == 'form-divider' %}

{% include 'form/partials/divider.html' %}

{% elif model_field.type == 'form-step' %}

{% include 'form/partials/step.html' %}

{% elif model_field.type == 'hidden-field' %}

{% include 'form/partials/hidden.html' %}

{% elif model_field.type == 'honeypot-field' %}

{% include 'form/partials/text_field.html' %}

{% endif %}