<div class="{% spaceless %}form-field form-field-type-{{model_field.type}} 
{% if model_field.extra_css_classes %}{{model_field.extra_css_classes}} {% endif %}
{% endspaceless %}">

{{model_field.content|safe}}

</div>