from django.conf.urls import patterns, url, include

urlpatterns = patterns('',
	
	# url(r'^tags/(?P<path>[\w-]+)/$', PageTagView.as_view(), name='pages_tag'),
	# url(r'^(?P<path>[-_\/\w]*)$', PageDetail.as_view(), name="pages_page"),
	# url(r'^(?P<path>.*)$', PageDetail.as_view(), name="pages_page"),
)